@@ -18,38 +18,38 @@ module Main
     , main
     ) where
 
-import           Test.Chell
+import Test.Tasty
 
-import           DBusTests.Address
-import           DBusTests.BusName
-import           DBusTests.Client
-import           DBusTests.ErrorName
-import           DBusTests.Integration
-import           DBusTests.InterfaceName
-import           DBusTests.Introspection
-import           DBusTests.MemberName
-import           DBusTests.Message
-import           DBusTests.ObjectPath
-import           DBusTests.Serialization
-import           DBusTests.Socket
-import           DBusTests.Signature
-import           DBusTests.Transport
-import           DBusTests.Variant
-import           DBusTests.Wire
+import DBusTests.Address
+import DBusTests.BusName
+import DBusTests.Client
+import DBusTests.ErrorName
+import DBusTests.Integration
+import DBusTests.InterfaceName
+import DBusTests.Introspection
+import DBusTests.MemberName
+import DBusTests.Message
+import DBusTests.ObjectPath
+import DBusTests.Serialization
+import DBusTests.Socket
+import DBusTests.Signature
+import DBusTests.Transport
+import DBusTests.Variant
+import DBusTests.Wire
 
 -- import all dbus modules here to ensure they show up in the coverage report,
 -- even if not tested.
-import           DBus ()
-import           DBus.Client ()
-import           DBus.Internal.Address ()
-import           DBus.Internal.Message ()
-import           DBus.Internal.Types ()
-import           DBus.Internal.Wire ()
-import           DBus.Introspection ()
-import           DBus.Socket ()
+import DBus ()
+import DBus.Client ()
+import DBus.Internal.Address ()
+import DBus.Internal.Message ()
+import DBus.Internal.Types ()
+import DBus.Internal.Wire ()
+import DBus.Introspection ()
+import DBus.Socket ()
 
-tests :: [Suite]
-tests =
+tests :: TestTree
+tests = testGroup "dbus"
     [ test_Address
     , test_BusName
     , test_Client
@@ -69,4 +69,4 @@ tests =
     ]
 
 main :: IO ()
-main = Test.Chell.defaultMain tests
+main = defaultMain tests
