@@ -1,5 +1,3 @@
-{-# LANGUAGE TemplateHaskell #-}
-
 -- Copyright (C) 2010-2012 John Millikin <john@john-millikin.com>
 --
 -- This program is free software: you can redistribute it and/or modify
@@ -17,16 +15,18 @@
 
 module DBusTests.Signature (test_Signature) where
 
-import           Test.Chell
-import           Test.Chell.QuickCheck
-import           Test.QuickCheck hiding ((.&.), property)
+import Data.Maybe
+import Test.QuickCheck
+import Test.Tasty
+import Test.Tasty.HUnit
+import Test.Tasty.QuickCheck
 
-import           DBus
+import DBus
 
-import           DBusTests.Util
+import DBusTests.Util
 
-test_Signature :: Suite
-test_Signature = suite "Signature"
+test_Signature :: TestTree
+test_Signature = testGroup "Signature"
     [ test_BuildSignature
     , test_ParseSignature
     , test_ParseInvalid
@@ -35,77 +35,80 @@ test_Signature = suite "Signature"
     , test_ShowType
     ]
 
-test_BuildSignature :: Test
-test_BuildSignature = property "signature" prop where
+test_BuildSignature :: TestTree
+test_BuildSignature = testProperty "signature" prop where
     prop = forAll gen_SignatureTypes check
     check types = case signature types of
         Nothing -> False
         Just sig -> signatureTypes sig == types
 
-test_ParseSignature :: Test
-test_ParseSignature = property "parseSignature" prop where
+test_ParseSignature :: TestTree
+test_ParseSignature = testProperty "parseSignature" prop where
     prop = forAll gen_SignatureString check
     check (s, types) = case parseSignature s of
         Nothing -> False
         Just sig -> signatureTypes sig == types
 
-test_ParseInvalid :: Test
-test_ParseInvalid = assertions "parse-invalid" $ do
+test_ParseInvalid :: TestTree
+test_ParseInvalid = testCase "parse-invalid" $ do
     -- at most 255 characters
-    $expect (just (parseSignature (replicate 254 'y')))
-    $expect (just (parseSignature (replicate 255 'y')))
-    $expect (nothing (parseSignature (replicate 256 'y')))
+    assertBool "valid parse failed" $
+        isJust (parseSignature (replicate 254 'y'))
+    assertBool "valid parse failed" $
+        isJust (parseSignature (replicate 255 'y'))
+    Nothing @=? parseSignature (replicate 256 'y')
 
     -- length also enforced by 'signature'
-    $expect (just (signature (replicate 255 TypeWord8)))
-    $expect (nothing (signature (replicate 256 TypeWord8)))
+    assertBool "valid parse failed" $
+        isJust (signature (replicate 255 TypeWord8))
+    Nothing @=? signature (replicate 256 TypeWord8)
 
     -- struct code
-    $expect (nothing (parseSignature "r"))
+    Nothing @=? parseSignature "r"
 
     -- empty struct
-    $expect (nothing (parseSignature "()"))
-    $expect (nothing (signature [TypeStructure []]))
+    Nothing @=? parseSignature "()"
+    Nothing @=? signature [TypeStructure []]
 
     -- dict code
-    $expect (nothing (parseSignature "e"))
+    Nothing @=? parseSignature "e"
 
     -- non-atomic dict key
-    $expect (nothing (parseSignature "a{vy}"))
-    $expect (nothing (signature [TypeDictionary TypeVariant TypeVariant]))
+    Nothing @=? parseSignature "a{vy}"
+    Nothing @=? signature [TypeDictionary TypeVariant TypeVariant]
 
-test_FormatSignature :: Test
-test_FormatSignature = property "formatSignature" prop where
+test_FormatSignature :: TestTree
+test_FormatSignature = testProperty "formatSignature" prop where
     prop = forAll gen_SignatureString check
     check (s, _) = let
         Just sig = parseSignature s
         in formatSignature sig == s
 
-test_IsAtom :: Test
-test_IsAtom = assertions "IsAtom" $ do
+test_IsAtom :: TestTree
+test_IsAtom = testCase "IsAtom" $ do
     let Just sig = signature []
     assertAtom TypeSignature sig
 
-test_ShowType :: Test
-test_ShowType = assertions "show-type" $ do
-    $expect (equal "Bool" (show TypeBoolean))
-    $expect (equal "Bool" (show TypeBoolean))
-    $expect (equal "Word8" (show TypeWord8))
-    $expect (equal "Word16" (show TypeWord16))
-    $expect (equal "Word32" (show TypeWord32))
-    $expect (equal "Word64" (show TypeWord64))
-    $expect (equal "Int16" (show TypeInt16))
-    $expect (equal "Int32" (show TypeInt32))
-    $expect (equal "Int64" (show TypeInt64))
-    $expect (equal "Double" (show TypeDouble))
-    $expect (equal "UnixFd" (show TypeUnixFd))
-    $expect (equal "String" (show TypeString))
-    $expect (equal "Signature" (show TypeSignature))
-    $expect (equal "ObjectPath" (show TypeObjectPath))
-    $expect (equal "Variant" (show TypeVariant))
-    $expect (equal "[Word8]" (show (TypeArray TypeWord8)))
-    $expect (equal "Dict Word8 (Dict Word8 Word8)" (show (TypeDictionary TypeWord8 (TypeDictionary TypeWord8 TypeWord8))))
-    $expect (equal "(Word8, Word16)" (show (TypeStructure [TypeWord8, TypeWord16])))
+test_ShowType :: TestTree
+test_ShowType = testCase "show-type" $ do
+    "Bool" @=? show TypeBoolean
+    "Bool" @=? show TypeBoolean
+    "Word8" @=? show TypeWord8
+    "Word16" @=? show TypeWord16
+    "Word32" @=? show TypeWord32
+    "Word64" @=? show TypeWord64
+    "Int16" @=? show TypeInt16
+    "Int32" @=? show TypeInt32
+    "Int64" @=? show TypeInt64
+    "Double" @=? show TypeDouble
+    "UnixFd" @=? show TypeUnixFd
+    "String" @=? show TypeString
+    "Signature" @=? show TypeSignature
+    "ObjectPath" @=? show TypeObjectPath
+    "Variant" @=? show TypeVariant
+    "[Word8]" @=? show (TypeArray TypeWord8)
+    "Dict Word8 (Dict Word8 Word8)" @=? show (TypeDictionary TypeWord8 (TypeDictionary TypeWord8 TypeWord8))
+    "(Word8, Word16)" @=? show (TypeStructure [TypeWord8, TypeWord16])
 
 gen_SignatureTypes :: Gen [Type]
 gen_SignatureTypes = do
