@@ -1,5 +1,4 @@
 {-# LANGUAGE OverloadedStrings #-}
-{-# LANGUAGE TemplateHaskell #-}
 
 -- Copyright (C) 2012 John Millikin <john@john-millikin.com>
 --
@@ -18,62 +17,65 @@
 
 module DBusTests.Transport (test_Transport) where
 
-import           Test.Chell
-
-import           Control.Concurrent
-import           Control.Monad.IO.Class (MonadIO, liftIO)
+import Control.Concurrent
+import Control.Monad.Extra (unlessM)
+import Control.Monad.IO.Class (liftIO)
+import Control.Monad.Trans.Resource
+import Data.Function (fix)
+import Data.List (isInfixOf)
+import Network.Socket.ByteString (sendAll, recv)
+import System.Directory (getTemporaryDirectory, removeFile)
+import Test.Tasty
+import Test.Tasty.HUnit
 import qualified Data.ByteString
-import           Data.Function (fix)
-import           Data.List (isPrefixOf)
 import qualified Data.Map as Map
-import qualified Network as N
 import qualified Network.Socket as NS
-import           Network.Socket.ByteString (sendAll, recv)
-import           System.Directory (getTemporaryDirectory, removeFile)
 
-import           DBus
-import           DBus.Transport
+import DBus
+import DBus.Transport
 
-import           DBusTests.Util
+import DBusTests.Util
 
-test_Transport :: Suite
-test_Transport = suite "Transport" $
-    suiteTests suite_TransportOpen ++
-    suiteTests suite_TransportListen ++
-    suiteTests suite_TransportAccept ++
-    [ test_TransportSendReceive
+test_Transport :: TestTree
+test_Transport = testGroup "Transport" $
+    [ suite_TransportOpen
+    , suite_TransportListen
+    , suite_TransportAccept
+    , test_TransportSendReceive
     , test_HandleLostConnection
     ]
 
-suite_TransportOpen :: Suite
-suite_TransportOpen = suite "transportOpen" $
+suite_TransportOpen :: TestTree
+suite_TransportOpen = testGroup "transportOpen" $
     [ test_OpenUnknown
-    ] ++ suiteTests suite_OpenUnix
-    ++ suiteTests suite_OpenTcp
+    , suite_OpenUnix
+    , suite_OpenTcp
+    ]
 
-suite_TransportListen :: Suite
-suite_TransportListen = suite "transportListen" $
+suite_TransportListen :: TestTree
+suite_TransportListen = testGroup "transportListen" $
     [ test_ListenUnknown
-    ] ++ suiteTests suite_ListenUnix
-    ++ suiteTests suite_ListenTcp
+    , suite_ListenUnix
+    , suite_ListenTcp
+    ]
 
-suite_TransportAccept :: Suite
-suite_TransportAccept = suite "transportAccept"
+suite_TransportAccept :: TestTree
+suite_TransportAccept = testGroup "transportAccept"
     [ test_AcceptSocket
     , test_AcceptSocketClosed
     ]
 
-test_OpenUnknown :: Test
-test_OpenUnknown = assertions "unknown" $ do
+test_OpenUnknown :: TestTree
+test_OpenUnknown = testCase "unknown" $ do
     let Just addr = address "noexist" Map.empty
-    $assert $ throwsEq
+    assertException
         ((transportError "Unknown address method: \"noexist\"")
             { transportErrorAddress = Just addr
             })
         (transportOpen socketTransportOptions addr)
 
-suite_OpenUnix :: Suite
-suite_OpenUnix = suite "unix"
+suite_OpenUnix :: TestTree
+suite_OpenUnix = testGroup "unix"
     [ test_OpenUnix_Path
     , test_OpenUnix_Abstract
     , test_OpenUnix_TooFew
@@ -81,82 +83,80 @@ suite_OpenUnix = suite "unix"
     , test_OpenUnix_NotListening
     ]
 
-test_OpenUnix_Path :: Test
-test_OpenUnix_Path = assertions "path" $ do
+test_OpenUnix_Path :: TestTree
+test_OpenUnix_Path = testCase "path" $ runResourceT $ do
     (addr, networkSocket) <- listenRandomUnixPath
-    afterTest (N.sClose networkSocket)
-
     fdcountBefore <- countFileDescriptors
 
     t <- liftIO (transportOpen socketTransportOptions addr)
     liftIO (transportClose t)
 
     fdcountAfter <- countFileDescriptors
-    $assert (equal fdcountBefore fdcountAfter)
-
-test_OpenUnix_Abstract :: Test
-test_OpenUnix_Abstract = assertions "abstract" $ do
-    (addr, networkSocket) <- listenRandomUnixAbstract
-    afterTest (N.sClose networkSocket)
+    liftIO (fdcountBefore @=? fdcountAfter)
 
+test_OpenUnix_Abstract :: TestTree
+test_OpenUnix_Abstract = testCase "abstract" $ runResourceT $ do
+    (addr, networkSocket, _) <- listenRandomUnixAbstract
     fdcountBefore <- countFileDescriptors
 
     t <- liftIO (transportOpen socketTransportOptions addr)
     liftIO (transportClose t)
 
     fdcountAfter <- countFileDescriptors
-    $assert (equal fdcountBefore fdcountAfter)
+    liftIO (fdcountBefore @=? fdcountAfter)
 
-test_OpenUnix_TooFew :: Test
-test_OpenUnix_TooFew = assertions "too-few" $ do
+test_OpenUnix_TooFew :: TestTree
+test_OpenUnix_TooFew = testCase "too-few" $ do
     fdcountBefore <- countFileDescriptors
 
     let Just addr = address "unix" Map.empty
-    $assert $ throwsEq
+    assertException
         ((transportError "One of 'path' or 'abstract' must be specified for the 'unix' transport.")
             { transportErrorAddress = Just addr
             })
         (transportOpen socketTransportOptions addr)
 
     fdcountAfter <- countFileDescriptors
-    $assert (equal fdcountBefore fdcountAfter)
+    fdcountBefore @=? fdcountAfter
 
-test_OpenUnix_TooMany :: Test
-test_OpenUnix_TooMany = assertions "too-many" $ do
+test_OpenUnix_TooMany :: TestTree
+test_OpenUnix_TooMany = testCase "too-many" $ do
     fdcountBefore <- countFileDescriptors
 
     let Just addr = address "unix" (Map.fromList
             [ ("path", "foo")
             , ("abstract", "bar")
             ])
-    $assert $ throwsEq
+    assertException
         ((transportError "Only one of 'path' or 'abstract' may be specified for the 'unix' transport.")
             { transportErrorAddress = Just addr
             })
         (transportOpen socketTransportOptions addr)
 
     fdcountAfter <- countFileDescriptors
-    $assert (equal fdcountBefore fdcountAfter)
+    fdcountBefore @=? fdcountAfter
 
-test_OpenUnix_NotListening :: Test
-test_OpenUnix_NotListening = assertions "not-listening" $ do
+test_OpenUnix_NotListening :: TestTree
+test_OpenUnix_NotListening = testCase "not-listening" $ runResourceT $ do
     fdcountBefore <- countFileDescriptors
 
-    (addr, networkSocket) <- listenRandomUnixAbstract
-    liftIO (NS.sClose networkSocket)
-    $assert $ throwsEq
-        ((transportError "connect: does not exist (Connection refused)")
-            { transportErrorAddress = Just addr
-            })
+    (addr, networkSocket, key) <- listenRandomUnixAbstract
+    release key
+
+    liftIO $ assertThrows
+        (\err -> and
+            [ "Connection refused" `isInfixOf` transportErrorMessage err
+            , transportErrorAddress err == Just addr
+            ])
         (transportOpen socketTransportOptions addr)
 
     fdcountAfter <- countFileDescriptors
-    $assert (equal fdcountBefore fdcountAfter)
+    liftIO (fdcountBefore @=? fdcountAfter)
 
-suite_OpenTcp :: Suite
-suite_OpenTcp = suite "tcp"
+suite_OpenTcp :: TestTree
+suite_OpenTcp = testGroup "tcp"
     [ test_OpenTcp_IPv4
-    , skipWhen noIPv6 test_OpenTcp_IPv6
+    , test_OpenTcp_IPv6
     , test_OpenTcp_Unknown
     , test_OpenTcp_NoPort
     , test_OpenTcp_InvalidPort
@@ -164,84 +164,80 @@ suite_OpenTcp = suite "tcp"
     , test_OpenTcp_NotListening
     ]
 
-test_OpenTcp_IPv4 :: Test
-test_OpenTcp_IPv4 = assertions "ipv4" $ do
-    (addr, networkSocket) <- listenRandomIPv4
-    afterTest (N.sClose networkSocket)
-
+test_OpenTcp_IPv4 :: TestTree
+test_OpenTcp_IPv4 = testCase "ipv4" $ runResourceT $ do
+    (addr, networkSocket, _) <- listenRandomIPv4
     fdcountBefore <- countFileDescriptors
 
     t <- liftIO (transportOpen socketTransportOptions addr)
     liftIO (transportClose t)
 
     fdcountAfter <- countFileDescriptors
-    $assert (equal fdcountBefore fdcountAfter)
+    liftIO (fdcountBefore @=? fdcountAfter)
 
-test_OpenTcp_IPv6 :: Test
-test_OpenTcp_IPv6 = assertions "ipv6" $ do
+test_OpenTcp_IPv6 :: TestTree
+test_OpenTcp_IPv6 = testCase "ipv6" $ unlessM noIPv6 $ runResourceT $ do
     (addr, networkSocket) <- listenRandomIPv6
-    afterTest (N.sClose networkSocket)
-
     fdcountBefore <- countFileDescriptors
 
     t <- liftIO (transportOpen socketTransportOptions addr)
     liftIO (transportClose t)
 
     fdcountAfter <- countFileDescriptors
-    $assert (equal fdcountBefore fdcountAfter)
+    liftIO (fdcountBefore @=? fdcountAfter)
 
-test_OpenTcp_Unknown :: Test
-test_OpenTcp_Unknown = assertions "unknown-family" $ do
+test_OpenTcp_Unknown :: TestTree
+test_OpenTcp_Unknown = testCase "unknown-family" $ do
     fdcountBefore <- countFileDescriptors
 
     let Just addr = address "tcp" (Map.fromList
             [ ("family", "noexist")
             , ("port", "1234")
             ])
-    $assert $ throwsEq
+    assertException
         ((transportError "Unknown socket family for TCP transport: \"noexist\"")
             { transportErrorAddress = Just addr
             })
         (transportOpen socketTransportOptions addr)
 
     fdcountAfter <- countFileDescriptors
-    $assert (equal fdcountBefore fdcountAfter)
+    fdcountBefore @=? fdcountAfter
 
-test_OpenTcp_NoPort :: Test
-test_OpenTcp_NoPort = assertions "no-port" $ do
+test_OpenTcp_NoPort :: TestTree
+test_OpenTcp_NoPort = testCase "no-port" $ do
     fdcountBefore <- countFileDescriptors
 
     let Just addr = address "tcp" (Map.fromList
             [ ("family", "ipv4")
             ])
-    $assert $ throwsEq
+    assertException
         ((transportError "TCP transport requires the `port' parameter.")
             { transportErrorAddress = Just addr
             })
         (transportOpen socketTransportOptions addr)
 
     fdcountAfter <- countFileDescriptors
-    $assert (equal fdcountBefore fdcountAfter)
+    fdcountBefore @=? fdcountAfter
 
-test_OpenTcp_InvalidPort :: Test
-test_OpenTcp_InvalidPort = assertions "invalid-port" $ do
+test_OpenTcp_InvalidPort :: TestTree
+test_OpenTcp_InvalidPort = testCase "invalid-port" $ do
     fdcountBefore <- countFileDescriptors
 
     let Just addr = address "tcp" (Map.fromList
             [ ("family", "ipv4")
             , ("port", "123456")
             ])
-    $assert $ throwsEq
+    assertException
         ((transportError "Invalid socket port for TCP transport: \"123456\"")
             { transportErrorAddress = Just addr
             })
         (transportOpen socketTransportOptions addr)
 
     fdcountAfter <- countFileDescriptors
-    $assert (equal fdcountBefore fdcountAfter)
+    fdcountBefore @=? fdcountAfter
 
-test_OpenTcp_NoUsableAddresses :: Test
-test_OpenTcp_NoUsableAddresses = assertions "no-usable-addresses" $ do
+test_OpenTcp_NoUsableAddresses :: TestTree
+test_OpenTcp_NoUsableAddresses = testCase "no-usable-addresses" $ do
     fdcountBefore <- countFileDescriptors
 
     let Just addr = address "tcp" (Map.fromList
@@ -249,35 +245,36 @@ test_OpenTcp_NoUsableAddresses = assertions "no-usable-addresses" $ do
             , ("port", "1234")
             , ("host", "256.256.256.256")
             ])
-    $assert $ throws
+    assertThrows
         (\err -> and
-            [ "getAddrInfo: does not exist" `isPrefixOf` transportErrorMessage err
+            [ "getAddrInfo" `isInfixOf` transportErrorMessage err
+            , "does not exist" `isInfixOf` transportErrorMessage err
             , transportErrorAddress err == Just addr
             ])
         (transportOpen socketTransportOptions addr)
 
     fdcountAfter <- countFileDescriptors
-    $assert (equal fdcountBefore fdcountAfter)
+    fdcountBefore @=? fdcountAfter
 
-test_OpenTcp_NotListening :: Test
-test_OpenTcp_NotListening = assertions "too-many" $ do
+test_OpenTcp_NotListening :: TestTree
+test_OpenTcp_NotListening = testCase "too-many" $ runResourceT $ do
     fdcountBefore <- countFileDescriptors
 
-    (addr, networkSocket) <- listenRandomIPv4
-    liftIO (NS.sClose networkSocket)
-    $assert $ throwsEq
-        ((transportError "connect: does not exist (Connection refused)")
-            { transportErrorAddress = Just addr
-            })
+    (addr, networkSocket, key) <- listenRandomIPv4
+    release key
+    liftIO $ assertThrows
+        (\err -> and
+            [ "Connection refused" `isInfixOf` transportErrorMessage err
+            , transportErrorAddress err == Just addr
+            ])
         (transportOpen socketTransportOptions addr)
 
     fdcountAfter <- countFileDescriptors
-    $assert (equal fdcountBefore fdcountAfter)
+    liftIO (fdcountBefore @=? fdcountAfter)
 
-test_TransportSendReceive :: Test
-test_TransportSendReceive = assertions "send-receive" $ do
-    (addr, networkSocket) <- listenRandomIPv4
-    afterTest (N.sClose networkSocket)
+test_TransportSendReceive :: TestTree
+test_TransportSendReceive = testCase "send-receive" $ runResourceT $ do
+    (addr, networkSocket, _) <- listenRandomIPv4
 
     -- a simple echo server, which sends back anything it receives.
     _ <- liftIO $ forkIO $ do
@@ -290,8 +287,9 @@ test_TransportSendReceive = assertions "send-receive" $ do
                     sendAll s bytes
                     loop
 
-    t <- liftIO (transportOpen socketTransportOptions addr)
-    afterTest (transportClose t)
+    (_, t) <- allocate
+        (transportOpen socketTransportOptions addr)
+        transportClose
 
     -- small chunks of data are combined
     do
@@ -300,7 +298,7 @@ test_TransportSendReceive = assertions "send-receive" $ do
         liftIO (transportPut t "2")
         liftIO (transportPut t "3")
         bytes <- liftIO (readMVar var)
-        $assert (equal bytes "123")
+        liftIO (bytes @?= "123")
 
     -- large chunks of data are read in full
     do
@@ -308,35 +306,35 @@ test_TransportSendReceive = assertions "send-receive" $ do
         var <- forkVar (transportGet t (4096 * 100))
         liftIO (transportPut t sentBytes)
         bytes <- liftIO (readMVar var)
-        $assert (equal bytes sentBytes)
+        liftIO (bytes @?= sentBytes)
 
-test_HandleLostConnection :: Test
-test_HandleLostConnection = assertions "handle-lost-connection" $ do
-    (addr, networkSocket) <- listenRandomIPv4
-    afterTest (N.sClose networkSocket)
+test_HandleLostConnection :: TestTree
+test_HandleLostConnection = testCase "handle-lost-connection" $ runResourceT $ do
+    (addr, networkSocket, _) <- listenRandomIPv4
 
     _ <- liftIO $ forkIO $ do
         (s, _) <- NS.accept networkSocket
         sendAll s "123"
         NS.sClose s
 
-    t <- liftIO (transportOpen socketTransportOptions addr)
-    afterTest (transportClose t)
+    (_, t) <- allocate
+        (transportOpen socketTransportOptions addr)
+        transportClose
 
     bytes <- liftIO (transportGet t 4)
-    $assert (equal bytes "123")
+    liftIO (bytes @?= "123")
 
-test_ListenUnknown :: Test
-test_ListenUnknown = assertions "unknown" $ do
+test_ListenUnknown :: TestTree
+test_ListenUnknown = testCase "unknown" $ do
     let Just addr = address "noexist" Map.empty
-    $assert $ throwsEq
+    assertException
         ((transportError "Unknown address method: \"noexist\"")
             { transportErrorAddress = Just addr
             })
         (transportListen socketTransportOptions addr)
 
-suite_ListenUnix :: Suite
-suite_ListenUnix = suite "unix"
+suite_ListenUnix :: TestTree
+suite_ListenUnix = testGroup "unix"
     [ test_ListenUnix_Path
     , test_ListenUnix_Abstract
     , test_ListenUnix_Tmpdir
@@ -345,196 +343,205 @@ suite_ListenUnix = suite "unix"
     , test_ListenUnix_InvalidBind
     ]
 
-test_ListenUnix_Path :: Test
-test_ListenUnix_Path = assertions "path" $ do
-    path <- liftIO getTempPath
+test_ListenUnix_Path :: TestTree
+test_ListenUnix_Path = testCase "path" $ runResourceT $ do
+    (_, path) <- allocate getTempPath removeFile
     let Just addr = address "unix" (Map.fromList
             [ ("path", path)
             ])
-    l <- liftIO (transportListen socketTransportOptions addr)
-    afterTest (transportListenerClose l)
-    afterTest (removeFile path)
+    (_, l) <- allocate
+        (transportListen socketTransportOptions addr)
+        transportListenerClose
 
     -- listener address is random, so it can't be checked directly.
     let addrParams = addressParameters (transportListenerAddress l)
-    $expect (sameItems (Map.keys addrParams) ["path", "guid"])
-    $expect (equal (Map.lookup "path" addrParams) (Just path))
+    liftIO (Map.keys addrParams @=? ["guid", "path"])
+    liftIO (Map.lookup "path" addrParams @?= Just path)
 
-test_ListenUnix_Abstract :: Test
-test_ListenUnix_Abstract = assertions "abstract" $ do
+test_ListenUnix_Abstract :: TestTree
+test_ListenUnix_Abstract = testCase "abstract" $ runResourceT $ do
     path <- liftIO getTempPath
     let Just addr = address "unix" (Map.fromList
             [ ("abstract", path)
             ])
-    l <- liftIO (transportListen socketTransportOptions addr)
-    afterTest (transportListenerClose l)
+    (_, l) <- allocate
+        (transportListen socketTransportOptions addr)
+        transportListenerClose
 
     -- listener address is random, so it can't be checked directly.
     let addrParams = addressParameters (transportListenerAddress l)
-    $expect (sameItems (Map.keys addrParams) ["abstract", "guid"])
-    $expect (equal (Map.lookup "abstract" addrParams) (Just path))
+    liftIO (Map.keys addrParams @?= ["abstract", "guid"])
+    liftIO (Map.lookup "abstract" addrParams @?= Just path)
 
-test_ListenUnix_Tmpdir :: Test
-test_ListenUnix_Tmpdir = assertions "tmpdir" $ do
+test_ListenUnix_Tmpdir :: TestTree
+test_ListenUnix_Tmpdir = testCase "tmpdir" $ runResourceT $ do
     tmpdir <- liftIO getTemporaryDirectory
     let Just addr = address "unix" (Map.fromList
             [ ("tmpdir", tmpdir)
             ])
-    l <- liftIO (transportListen socketTransportOptions addr)
-    afterTest (transportListenerClose l)
+    (_, l) <- allocate
+        (transportListen socketTransportOptions addr)
+        transportListenerClose
 
     -- listener address is random, so it can't be checked directly.
     let addrKeys = Map.keys (addressParameters (transportListenerAddress l))
-    $expect ("path" `elem` addrKeys || "abstract" `elem` addrKeys)
+    liftIO $ assertBool "invalid keys"
+        ("path" `elem` addrKeys || "abstract" `elem` addrKeys)
 
-test_ListenUnix_TooFew :: Test
-test_ListenUnix_TooFew = assertions "too-few" $ do
+test_ListenUnix_TooFew :: TestTree
+test_ListenUnix_TooFew = testCase "too-few" $ do
     let Just addr = address "unix" Map.empty
-    $assert $ throwsEq
+    assertException
         ((transportError "One of 'abstract', 'path', or 'tmpdir' must be specified for the 'unix' transport.")
             { transportErrorAddress = Just addr
             })
         (transportListen socketTransportOptions addr)
 
-test_ListenUnix_TooMany :: Test
-test_ListenUnix_TooMany = assertions "too-many" $ do
+test_ListenUnix_TooMany :: TestTree
+test_ListenUnix_TooMany = testCase "too-many" $ do
     let Just addr = address "unix" (Map.fromList
             [ ("path", "foo")
             , ("abstract", "bar")
             ])
-    $assert $ throwsEq
+    assertException
         ((transportError "Only one of 'abstract', 'path', or 'tmpdir' may be specified for the 'unix' transport.")
             { transportErrorAddress = Just addr
             })
         (transportListen socketTransportOptions addr)
 
-test_ListenUnix_InvalidBind :: Test
-test_ListenUnix_InvalidBind = assertions "invalid-bind" $ do
+test_ListenUnix_InvalidBind :: TestTree
+test_ListenUnix_InvalidBind = testCase "invalid-bind" $ do
     fdcountBefore <- countFileDescriptors
 
     let Just addr = address "unix" (Map.fromList
             [ ("path", "/")
             ])
-    $assert $ throwsEq
-        ((transportError "bind: resource busy (Address already in use)")
-            { transportErrorAddress = Just addr
-            })
+    assertThrows
+        (\err -> and
+            [ "Address already in use" `isInfixOf` transportErrorMessage err
+            , transportErrorAddress err == Just addr
+            ])
         (transportListen socketTransportOptions addr)
 
     fdcountAfter <- countFileDescriptors
-    $assert (equal fdcountBefore fdcountAfter)
+    fdcountBefore @=? fdcountAfter
 
-suite_ListenTcp :: Suite
-suite_ListenTcp = suite "tcp"
+suite_ListenTcp :: TestTree
+suite_ListenTcp = testGroup "tcp"
     [ test_ListenTcp_IPv4
-    , skipWhen noIPv6 test_ListenTcp_IPv6
+    , test_ListenTcp_IPv6
     , test_ListenTcp_Unknown
     , test_ListenTcp_InvalidPort
     , test_ListenTcp_InvalidBind
     ]
 
-test_ListenTcp_IPv4 :: Test
-test_ListenTcp_IPv4 = assertions "ipv4" $ do
+test_ListenTcp_IPv4 :: TestTree
+test_ListenTcp_IPv4 = testCase "ipv4" $ runResourceT $ do
     let Just addr = address "tcp" (Map.fromList
             [ ("family", "ipv4")
             ])
-    l <- liftIO (transportListen socketTransportOptions addr)
-    afterTest (transportListenerClose l)
+    (_, l) <- allocate
+        (transportListen socketTransportOptions addr)
+        transportListenerClose
 
     let params = addressParameters (transportListenerAddress l)
-    $expect (equal (Map.lookup "family" params) (Just "ipv4"))
-    $expect ("port" `elem` Map.keys params)
+    liftIO (Map.lookup "family" params @?= Just "ipv4")
+    liftIO $ assertBool "no port" ("port" `elem` Map.keys params)
 
-test_ListenTcp_IPv6 :: Test
-test_ListenTcp_IPv6 = assertions "ipv6" $ do
+test_ListenTcp_IPv6 :: TestTree
+test_ListenTcp_IPv6 = testCase "ipv6" $ unlessM noIPv6 $ runResourceT $ do
     let Just addr = address "tcp" (Map.fromList
             [ ("family", "ipv6")
             ])
-    l <- liftIO (transportListen socketTransportOptions addr)
-    afterTest (transportListenerClose l)
+    (_, l) <- allocate
+        (transportListen socketTransportOptions addr)
+        transportListenerClose
 
     let params = addressParameters (transportListenerAddress l)
-    $expect (equal (Map.lookup "family" params) (Just "ipv6"))
-    $expect ("port" `elem` Map.keys params)
+    liftIO (Map.lookup "family" params @?= Just "ipv6")
+    liftIO $ assertBool "no port" ("port" `elem` Map.keys params)
 
-test_ListenTcp_Unknown :: Test
-test_ListenTcp_Unknown = assertions "unknown-family" $ do
+test_ListenTcp_Unknown :: TestTree
+test_ListenTcp_Unknown = testCase "unknown-family" $ do
     let Just addr = address "tcp" (Map.fromList
             [ ("family", "noexist")
             , ("port", "1234")
             ])
-    $assert $ throwsEq
+    assertException
         ((transportError "Unknown socket family for TCP transport: \"noexist\"")
             { transportErrorAddress = Just addr
             })
         (transportListen socketTransportOptions addr)
 
-test_ListenTcp_InvalidPort :: Test
-test_ListenTcp_InvalidPort = assertions "invalid-port" $ do
+test_ListenTcp_InvalidPort :: TestTree
+test_ListenTcp_InvalidPort = testCase "invalid-port" $ do
     let Just addr = address "tcp" (Map.fromList
             [ ("family", "ipv4")
             , ("port", "123456")
             ])
-    $assert $ throwsEq
+    assertException
         ((transportError "Invalid socket port for TCP transport: \"123456\"")
             { transportErrorAddress = Just addr
             })
         (transportListen socketTransportOptions addr)
 
-test_ListenTcp_InvalidBind :: Test
-test_ListenTcp_InvalidBind = assertions "invalid-bind" $ do
+test_ListenTcp_InvalidBind :: TestTree
+test_ListenTcp_InvalidBind = testCase "invalid-bind" $ do
     fdcountBefore <- countFileDescriptors
 
     let Just addr = address "tcp" (Map.fromList
             [ ("family", "ipv4")
             , ("port", "1")
             ])
-    $assert $ throwsEq
-        ((transportError "bind: permission denied (Permission denied)")
-            { transportErrorAddress = Just addr
-            })
+    assertThrows
+        (\err -> and
+            [ "Permission denied" `isInfixOf` transportErrorMessage err
+            , transportErrorAddress err == Just addr
+            ])
         (transportListen socketTransportOptions addr)
 
     fdcountAfter <- countFileDescriptors
-    $assert (equal fdcountBefore fdcountAfter)
+    fdcountBefore @=? fdcountAfter
 
-test_AcceptSocket :: Test
-test_AcceptSocket = assertions "socket" $ do
+test_AcceptSocket :: TestTree
+test_AcceptSocket = testCase "socket" $ runResourceT $ do
     path <- liftIO getTempPath
     let Just addr = address "unix" (Map.fromList
             [ ("abstract", path)
             ])
-    listener <- liftIO (transportListen socketTransportOptions addr)
-    afterTest (transportListenerClose listener)
+    (_, listener) <- allocate
+        (transportListen socketTransportOptions addr)
+        transportListenerClose
 
     acceptedVar <- forkVar (transportAccept listener)
     openedVar <- forkVar (transportOpen socketTransportOptions addr)
 
-    accepted <- liftIO (readMVar acceptedVar)
-    opened <- liftIO (readMVar openedVar)
-    afterTest (transportClose accepted)
-    afterTest (transportClose opened)
+    (_, accepted) <- allocate (readMVar acceptedVar) transportClose
+    (_, opened) <- allocate (readMVar openedVar) transportClose
 
     liftIO (transportPut opened "testing")
 
     bytes <- liftIO (transportGet accepted 7)
 
-    $expect (equal bytes "testing")
+    liftIO (bytes @?= "testing")
 
-test_AcceptSocketClosed :: Test
-test_AcceptSocketClosed = assertions "socket-closed" $ do
-    path <- liftIO getTempPath
+test_AcceptSocketClosed :: TestTree
+test_AcceptSocketClosed = testCase "socket-closed" $ do
+    path <- getTempPath
     let Just addr = address "unix" (Map.fromList
             [ ("abstract", path)
             ])
-    listener <- liftIO (transportListen socketTransportOptions addr)
+    listener <- transportListen socketTransportOptions addr
     let listeningAddr = transportListenerAddress listener
-    liftIO (transportListenerClose listener)
+    transportListenerClose listener
 
-    $assert $ throwsEq
-        ((transportError "user error (accept: can't perform accept on socket ((AF_UNIX,Stream,0)) in status Closed)")
-            { transportErrorAddress = Just listeningAddr
-            })
+    assertThrows
+        (\err -> and
+            [ "accept" `isInfixOf` transportErrorMessage err
+            , "socket" `isInfixOf` transportErrorMessage err
+            , "Closed" `isInfixOf` transportErrorMessage err
+            , transportErrorAddress err == Just listeningAddr
+            ])
         (transportAccept listener)
 
 socketTransportOptions :: TransportOptions SocketTransport
